clc;
clear all;

%%% Call for Parameters
[alpha beta g h nominalX ] = state5paramFermen();  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Nominal System:              Ad*Yd = b - Ai*Yi                     %
% Perturbed nominal System:    Adp*Ydp = bp - Aip*Yip                %
% Perturbed synthetic System:  Adp*Ydp_syn = bp - Aip*(Yip + f)      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ n , k ] = size(g); % n Number of Dependent variables
m = k - n;           %  (k=n+m) Number of Independent variables

F = zeros(n);        % Control Parameter Matrix
f = zeros(m,1);      % Control Paramter Vector

Ad = g(: , 1:n) - h (: , 1:n);       %%% System Matrix (Kinetic parameters)
Ai = g ( : , n+1:k ) - h ( : , n+1:k );   %%%   Inteaction Matrix  %%%
b = log(beta./alpha);


Xi = nominalX(n+1:k);  % Nominal value of independent variables
Yi = log(Xi);

Yd = (inv(Ad))*(b-Ai*Yi);
Xd = exp(Yd);          % Nominal value of dependent variables
%%%%%%%%%%%%%%%%% Update nominalX  %%%%%%%%%%%%%%%%%%%%%%%%%%%%
nominalX = [Xd;Xi];

Xsr=1*Xd;           % Xsr is the desired steady state

Xsr(3)=1.2*Xd(3);   % x3 and x4 is steared to 120%.
Xsr(4)=1.2*Xd(4);

Ysr=log(Xsr);

% Steady-state sensitivity of nominal network wrt b 

[S_nom_b] = NL_Sensitivity_b(alpha, beta, g, h, nominalX,F,f);
sen_nom_b = norm(S_nom_b,'fro');
sen_nom_b_MAX = max(max(abs(S_nom_b)));

% Steady-state sensitivity of nominal network wrt g and h

[S_nom_gh] = NL_Sensitivity_gh(alpha, beta, g, h, nominalX,F,f);
sen_nom_gh = norm(S_nom_gh,'fro');
sen_nom_gh_MAX = max(max(abs(S_nom_gh)));



dw = 0.1;

WValue = 0.0 : dw : 1;
KValue = [1 5 10];

e_ssB = zeros(length(WValue),length(KValue));
REL_SS_ERROR = zeros(length(WValue),length(KValue));

SbB =   zeros(length(WValue),length(KValue));
SYiB =  zeros(length(WValue),length(KValue));
SAiB =  zeros(length(WValue),length(KValue));
SAdB =  zeros(length(WValue),length(KValue));

NormF = zeros(length(WValue),length(KValue));
Normf = zeros(length(WValue),length(KValue));

%%%%%%% FOR TESTING PURPOSE %%%%%%%%%%%%%%%%%
% ss1 = zeros(length(WValue),length(KValue));
% ss2 = zeros(length(WValue),length(KValue));
% ss3 = zeros(length(WValue),length(KValue));
% ss4 = zeros(length(WValue),length(KValue));
% ss5 = zeros(length(WValue),length(KValue));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



for wIter = 1 : 1 : length(WValue)
    
    for kIter = 1 : 1 : length(KValue)
        [P, F, f, s1, s2, s3 ,s4, s5 ]= findSynthParams(Ai,Ad, Yi, Ysr, Yd, WValue(wIter), KValue(kIter) );

        AdF_INV = inv(Ad+F);

        Yd_reached=AdF_INV*(b-Ai*(Yi+f));

        Xd_final=exp(Yd_reached);


        % Checking stability from the location of eigen values
        eigValues = [(eig(Ad)) (eig(Ad+F))];



        %%%%%%%% Synthetic Network Parameter Calculation
        %% Synthetic network's Gsn and Hsn  for rowh1=0.5 %%%%%
        row_g = size(g,1);
        Gsn = g;
        Hsn = h; 
        for k = 1 : row_g
            for p = 1 : row_g
                if Gsn(k,p) == 0
                    Hsn(k,p) = Hsn(k,p) - F(k,p);
                else
                    Gsn(k,p) = Gsn(k,p) + F(k,p);
                end
            end
        end

        g = Gsn;
        h = Hsn;

        Xi_synth = exp(Yi+f);

        synthX = [Xd_final; Xi_synth];

        %%%%%%%%%% Steady-state sensitivity of synthetic network
        [S_syn_b] = NL_Sensitivity_b(alpha, beta, g, h, nominalX,F,f);
        sen_syn_b = norm(S_syn_b,'fro');
        sen_syn_b_MAX = max(max(abs(S_syn_b)));

        [S_syn_gh] = NL_Sensitivity_gh(alpha, beta, g, h, nominalX,F,f);
        sen_syn_gh = norm(S_syn_gh,'fro');
        sen_syn_gh_MAX = max(max(abs(S_syn_gh)));

        fi = exp(f); 
        alpha_synth = alpha.*1;
        beta_synth = beta.*1;
        X0 = Xd;
        Tstop = 5;
        
        %%%%% Simulation in Simulink %%%%%%%%%%

        %    open_system('Fermentation');               % Open Simulink(R) Model
        %    sim('Fermentation',Tstop);
        %    trajectory_plot2( a,'x1_x4_plot.eps');
        %    trajectory_plot3( a, 'x2_x3_x5_plot.eps');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %%%%%%% Relative steady-state error in percentage %%%%%%%%%
        REL_SS_ERROR(wIter,kIter) = norm(Xsr-Xd_final)*100/norm(Xsr);

        %%%%%%%%% Computation of bounds %%%%%%%%%%%%%%%%
        E=Ad*(Ysr-Yd);

        %%%%%%% FOR TESTING PURPOSE %%%%%%%%%%%%%%%%%
        % ss1(wIter,kIter) = s1;
        % ss2(wIter,kIter) = s2;
        % ss3(wIter,kIter) = s3;
        % ss4(wIter,kIter) = s4;
        % ss5(wIter,kIter) = s5;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        e_ssB(wIter,kIter) = norm(AdF_INV,2)*norm(E+F*Ysr + Ai*f,2)/norm(Ysr,2);
        
        
        SbB(wIter,kIter) = norm(AdF_INV,2);
        SYiB(wIter,kIter) = norm(AdF_INV,2)*norm(Ai,2);
        SAiB(wIter,kIter) = norm(AdF_INV,2)*norm((Yi+f),2);
        SAdB(wIter,kIter) = ((norm(AdF_INV,2))^2)*(norm(b,2)+(norm(Ai,2))*norm((Yi+f),2));
        
        

        NormF(wIter,kIter) = norm(F,2);
        Normf(wIter,kIter) = norm(f,2);

    end %%%%% end of while loop
    
end
%%%%%%%%%% Plot of various graphs %%%%%%%%%%
legendKValue1 = 'k=1';
legendKValue2 = 'k=5';
legendKValue3 = 'k=10';


outName='Ess_B.eps';
legendPosition='NorthEast';
S_or_NormBound_plot(WValue,e_ssB,legendKValue1,legendKValue2,legendKValue3,'Error Bound',outName,legendPosition);

figure
outName='Sb_B.eps';
legendPosition='NorthEast';
S_or_NormBound_plot(WValue,SbB,legendKValue1,legendKValue2,legendKValue3,'S_b^{B}',outName,legendPosition);

figure
outName='SYi_B.eps';
legendPosition='NorthEast';
S_or_NormBound_plot(WValue,SYiB,legendKValue1,legendKValue2,legendKValue3,'S_{YI}^{B}',outName,legendPosition);

figure
outName='SAi_B.eps';
legendPosition='NorthEast';
S_or_NormBound_plot(WValue,SAiB,legendKValue1,legendKValue2,legendKValue3,'S_{AI}^{B}',outName,legendPosition);

figure
outName='SAd_B.eps';
legendPosition='NorthEast';
S_or_NormBound_plot(WValue,SAdB,legendKValue1,legendKValue2,legendKValue3,'S_{AD}^{B}',outName,legendPosition);



figure
outName='norm_F.eps';
legendPosition='NorthWest';
S_or_NormBound_plot(WValue,NormF,legendKValue1,legendKValue2,legendKValue3,'||F||',outName,legendPosition);

figure
outName='norm_f.eps';
legendPosition='NorthWest';
S_or_NormBound_plot(WValue,Normf,legendKValue1,legendKValue2,legendKValue3,'||f||',outName,legendPosition);

%%%%%%% FOR TESTING PURPOSE %%%%%%%%%%%%%%%%%
% outName='s1.eps';
% legendPosition='NorthEast';
% S_or_NormBound_plot(WValue(2:end),ss1(2:end,:),legendKValue1,legendKValue2,legendKValue3,'s_1',outName,legendPosition);
% % 
% figure
% outName='s2.eps';
% legendPosition='West';
% S_or_NormBound_plot(WValue(2:end),ss2(2:end,:),legendKValue1,legendKValue2,legendKValue3,'s_2',outName,legendPosition);
% 
% figure
% outName='s3.eps';
% legendPosition='NorthEast';
% S_or_NormBound_plot(WValue(2:end),ss3(2:end,:),legendKValue1,legendKValue2,legendKValue3,'s_3',outName,legendPosition);
% 
% figure
% outName='s4.eps';
% legendPosition='NorthEast';
% S_or_NormBound_plot(WValue(1:end-1),ss4(1:end-1,:),legendKValue1,legendKValue2,legendKValue3,'s_4',outName,legendPosition);
% 
% figure
% outName='s5.eps';
% legendPosition='NorthEast';
% S_or_NormBound_plot(WValue(1:end-1),ss5(1:end-1,:),legendKValue1,legendKValue2,legendKValue3,'s_5',outName,legendPosition);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
return

%% End of function %%%%%%%%%%